#include "csprite.h"


/*

Pretty much the same as the HS Fortuna example.

In stead of UV cooordinates for the texture, I'm using ST coordinates...
Pretty much the same, I'm just more used to ST coordinates...

While UV coordiantes are 0 to pixelWidth, ST coordinates are 0.0 to 1.0
independent of the actual pixel width...

You can always convert with (startTextureX/TextureWidth, endTextureX/TextureWidth).
For example, your Texture is 256x256 and you want to use a 32x48 square starting at 20,80.
S1 = 20/256, S2 = (20+32)/256, T1 = 80/256, T2 = (80+48)/256...

Then just draw a triangle Strip like this:

		  1                  3
		Pos1 ------------- Pos2
		 +				  /	+
		 +	 /  SPRITE	/	+
		 + /				+
		Pos3 ------------- Pos4
         2					4

First triangle is Pos1,Pos3,Pos2
Second triangle is Pos2,Pos3,Pos4
*/	


// Initialize sprite.
CSprite::CSprite( float xPos, float yPos, int xTexture, int yTexture, int widthTexture, int heightTexture)
{
	m_width = widthTexture;
	m_height = heightTexture;
	m_xTexture = xTexture;
	m_yTexture = yTexture;
	m_fHalfW = (float)widthTexture/2.0;
	m_fHalfH = (float)heightTexture/2.0;
	m_fSize = (m_fHalfW + m_fHalfH)/2.0;
	m_xPos = xPos;
	m_yPos = yPos;
	m_isVisible = true;
	m_Red = 0x80;
	m_Green = 0x80;
	m_Blue = 0x80;
	m_Alpha = 0x80;
	m_Angle = 0.0;
	m_fScaleX = 1.0;
	m_fScaleY = 1.0;
	
	m_iCustom1 = 0;
	m_iCustom2 = 0;
	m_iCustom3 = 0;
	m_iCustom4 = 0;
	m_iCustom5 = 0;
	m_fCustom1 = 0.0;
	m_fCustom2 = 0.0;
	m_fCustom3 = 0.0;
	m_fCustom4 = 0.0;
	m_fCustom5 = 0.0;

	m_iAnimationDirectionTexture = 0;
	m_iAnimationFramesInAnimation = 0;
	m_iAnimationDirectionOfFrame = 1;
	m_iAnimationFramePos = 0;
	m_iAnimationFrameRepeat = 1;
	m_iAnimationFrameRepeatBackAndForth = 1;
	m_iAnimationFrameDelay = 0;
	m_iAnimationFrameDelayCounter = 0;
	
	setTexturePosUV( m_xTexture, m_xTexture + m_width, m_yTexture, m_yTexture + m_height);
	setPos(m_xPos, m_yPos);
}

// Clean up.
CSprite::~CSprite()
{
}

// Set Position.
// When we set the position, we also calcualte the four corners depending on size and rotation.
// This is the same as the HS Fortuna example.
void CSprite::setPos(float xPos, float yPos) {
	m_xPos = xPos;
	m_yPos = yPos;

	// Cosine and sine.
	float cosi = cosf(m_Angle);
	float sini = sinf(m_Angle);
		
	// Untransformed coordinates
	// tl = top left
	// bl = bottom left
	// tr = top right
	// br = bottom right
	float _tlX = -m_fHalfW, _tlY = -m_fHalfH;
	float _blX = -m_fHalfW, _blY =  m_fHalfH;
	float _trX =  m_fHalfW, _trY = -m_fHalfH;
	float _brX =  m_fHalfW, _brY =  m_fHalfH;

	// Rotate the coordinates
	float tlX  = _tlX * cosi*m_fScaleX - _tlY * sini*m_fScaleY;
	float tlY  = _tlY * cosi*m_fScaleY + _tlX * sini*m_fScaleX;

	float blX  = _blX * cosi*m_fScaleX - _blY * sini*m_fScaleY;
	float blY  = _blY * cosi*m_fScaleY + _blX * sini*m_fScaleX;

	float trX  = _trX * cosi*m_fScaleX - _trY * sini*m_fScaleY;
	float tr_Y  = _trY * cosi*m_fScaleY + _trX * sini*m_fScaleX;

	float brX = _brX * cosi*m_fScaleX - _brY * sini*m_fScaleY;
	float brY = _brY * cosi*m_fScaleY + _brX * sini*m_fScaleX;

	m_xPos1 = (int)(m_xPos + tlX);
	m_yPos1 = (int)(m_yPos + tlY);

	m_xPos2 = (int)(m_xPos + trX);
	m_yPos2 = (int)(m_yPos + tr_Y);

	m_xPos3 = (int)(m_xPos + blX);
	m_yPos3 = (int)(m_yPos + blY);

	m_xPos4 = (int)(m_xPos + brX);
	m_yPos4 = (int)(m_yPos + brY);

}

// We can also set all four coordinates of the sprite, this comes in handy if you don't want square sprites.
// This is being used in the example...
void CSprite::setPosAll(int xPos1, int yPos1, int xPos2, int yPos2, int xPos3, int yPos3, int xPos4, int yPos4) {
	m_xPos1 = xPos1;
	m_yPos1 = yPos1;
	m_xPos2 = xPos2;
	m_yPos2 = yPos2;
	m_xPos3 = xPos3;
	m_yPos3 = yPos3;
	m_xPos4 = xPos4;
	m_yPos4 = yPos4;
}

// Scale X
void CSprite::setScaleX(float scaleX) {
	m_fScaleX = scaleX;
	setPos(m_xPos, m_yPos);
}

// Scale Y
void CSprite::setScaleY(float scaleY) {
	m_fScaleY = scaleY;
	setPos(m_xPos, m_yPos);
}

// Rotation Angle
void CSprite::setAngle(float Angle) {
	m_Angle = Angle;
	setPos(m_xPos, m_yPos);
}

// Incremenat rotation Angle
void CSprite::addAngle(float Angle) {
	m_Angle += Angle;
	setPos(m_xPos, m_yPos);
}

// Get Angle of Sprite
float CSprite::getAngle(void) {
	return m_Angle;
}

// Get size of Sprite
float CSprite::getSize(void) {
	return m_fSize;
}

// Move sprite x, y positions.
void CSprite::movePos(float xPos, float yPos) {
	m_xPos += xPos;
	m_yPos += yPos;
	setPos(m_xPos, m_yPos);
}

void CSprite::moveDirection(float direction) {

	// Cosine and sine.
	float cosi = cosf(m_Angle-1.57079632679);
	float sini = sinf(m_Angle-1.57079632679);
	// Move in current direction...
	m_xPos += direction*cosi;
	m_yPos += direction*sini;
	setPos(m_xPos, m_yPos);
}

// Which part of the texture to use.
// Rememeber ST uses 0.0 to 1.0 in stead of pixel coordinates...
void CSprite::setTexturePosST(float s1, float s2, float t1, float t2) {
	m_s1 = s1;
	m_s2 = s2;
	m_t1 = t1;
	m_t2 = t2;
}

// Currently hardcoded for 256x256 textures !!!
void CSprite::setTexturePosUV(int u1, int u2, int v1, int v2) {
	m_s1 = ((float)(u1))/256.0;
	m_s2 = ((float)(u2))/256.0;
	m_t1 = ((float)(v1))/256.0;
	m_t2 = ((float)(v2))/256.0;
}

void CSprite::setTexturePos(int xTexturePos, int yTexturePos) {
	setTexturePosUV( m_xTexture + (m_width*xTexturePos), m_xTexture + m_width + (m_width*xTexturePos), m_yTexture + (m_height*yTexturePos), m_yTexture + m_height + (m_height*yTexturePos));
}

// Set Sprite color
void CSprite::setColor(int Red, int Green, int Blue, int Alpha) {
	m_Red = Red;
	m_Green = Green;
	m_Blue = Blue;
	m_Alpha = Alpha;
}

// Set sprite visibility.
void CSprite::setVisible(bool isVisible) {
	m_isVisible = isVisible;
}

// Is the sprite visible ???
bool CSprite::getVisible(void) {
	return m_isVisible;
}

// Get X position center of Sprite
float CSprite::getXPos(void) {
	return m_xPos;
}

// Get Y position center of Sprite
float CSprite::getYPos(void) {
	return m_yPos;
}

// Get X position top left corner.
int CSprite::getXPos1(void) {
	return m_xPos1;
}

// Get Y position top left corner.
int CSprite::getYPos1(void) {
	return m_yPos1;
}

// Get X position top right corner.
int CSprite::getXPos2(void) {
	return m_xPos2;
}

// Get Y position top right corner.
int CSprite::getYPos2(void) {
	return m_yPos2;
}

// Get X position bottom left corner.
int CSprite::getXPos3(void) {
	return m_xPos3;
}

// Get Y position bottom left corner.
int CSprite::getYPos3(void) {
	return m_yPos3;
}

// Get X position bottom right corner.
int CSprite::getXPos4(void) {
	return m_xPos4;
}

// Get Y position bottom right corner.
int CSprite::getYPos4(void) {
	return m_yPos4;
}

// Get X scale
float CSprite::getScaleX(void) {
	return m_fScaleX;
}

// Get Y scale
float CSprite::getScaleY(void) {
	return m_fScaleY;
}

// Collision test.
bool CSprite::inside(CSprite inSprite) {

	// Check only visible sprites...
	if (!m_isVisible || !inSprite.getVisible()) {
		return false;
	}

	float rad, sumX, sumY;
	rad = (inSprite.getSize() + m_fSize)*(inSprite.getSize() + m_fSize);
	sumX = (inSprite.getXPos() - m_xPos)*(inSprite.getXPos() - m_xPos);
	sumY = (inSprite.getYPos() - m_yPos)*(inSprite.getYPos() - m_yPos);
	if ((sumX+sumY) < rad) {
		return true;
	} else {
		return false;
	}
}


// Add Sprite to primitive memory.
void CSprite::addSpriteToMem(char *& pMem, int Zvalue) {

	// Only render visible sprites.
	if (!m_isVisible) {
		return;
	}

	// Since we are using Triangle Strips, we can only write one sprite at a time...
	// Previously we could use one GIFTag header for all our square sprites, this is not the case anymore.
	AddGIFTag(pMem, 1);
	addSpriteToGIFTag(pMem, Zvalue);
}


// Just add the GIF Tag to the beginning of the allocated Memory.
void CSprite::AddGIFTag(char *& pMem, int numSprites) {

		// Add a GIF Tag 
		sps2GIFTag_t * pGIFTag = (sps2GIFTag_t *) pMem;
 
	  	// Create the GIFTag  
  		pGIFTag->i128=0;       			// Blank it out.
	  	pGIFTag->s.NLOOP=numSprites;   	// How many sprites we are rendering.
  		pGIFTag->s.PRE=1;      			// Send our PRIM value to the GIF via this GIFTag.
	  	pGIFTag->s.EOP=1;      			// This is the end of the packets, no more GIFTags are to be expected after the sprites.
		pGIFTag->s.PRIM = GS_PRIM(PRIM_TRI_STRIP, 
							PRIM_IIP_FLAT, 
							PRIM_TME_ON,
							PRIM_FGE_OFF, 
							PRIM_ABE_ON, 
							PRIM_AA1_OFF, 
							PRIM_FST_STQ, 
							PRIM_CTXT_CONTEXT1, 
							PRIM_FIX_NOFIXDDA);
	  	pGIFTag->s.FLG=GIF_FLG_PACKED;	// Use packed format for data.

	 	pGIFTag->s.NREG = 9; 
		pGIFTag->s.REGS =    GIF_REG_RGBAQ | (GIF_REG_ST << 4) | (GIF_REG_XYZ2 << 8) | (GIF_REG_ST << 12) | (GIF_REG_XYZ2 << 16) | (GIF_REG_ST << 20) |
		(GIF_REG_XYZ2 << 24) | (GIF_REG_ST << 28) | (((uint64)GIF_REG_XYZ2) << 32);
		pGIFTag++;

		// Update memory pointer.
		pMem = (char *)pGIFTag;
}

// Render sprite.
void CSprite::addSpriteToGIFTag(char *& pMem, int Zvalue) {

	// pGIFReg is now pointing to the first memory position.
  	sps2GIFPackedRegister_t * pGIFReg = (sps2GIFPackedRegister_t *) pMem;

	AddColour(pGIFReg, m_Red, m_Green, m_Blue, m_Alpha);

	// Upper Left Corner.
	AddSTQ(pGIFReg, m_s1, m_t1, 1.0f);
	AddVertex(pGIFReg, m_xPos1, m_yPos1, Zvalue);

	// Upper Right Corner.
	AddSTQ(pGIFReg, m_s1, m_t2, 1.0f);
	AddVertex(pGIFReg, m_xPos3, m_yPos3, Zvalue);

	// Bottom Left Corner.
	AddSTQ(pGIFReg, m_s2, m_t1, 1.0f);
	AddVertex(pGIFReg, m_xPos2, m_yPos2, Zvalue);

	// Bottom Right Corner.
	AddSTQ(pGIFReg, m_s2, m_t2, 1.0f);
	AddVertex(pGIFReg, m_xPos4, m_yPos4, Zvalue);

	// Update memory pointer.
	pMem = (char *)pGIFReg;
}



// Custom Integers.
void CSprite::setCustomInt1(int value) {
	m_iCustom1 = value;
}
void CSprite::setCustomInt2(int value) {
	m_iCustom2 = value;
}
void CSprite::setCustomInt3(int value) {
	m_iCustom3 = value;
}
void CSprite::setCustomInt4(int value) {
	m_iCustom4 = value;
}
void CSprite::setCustomInt5(int value) {
	m_iCustom5 = value;
}
int CSprite::getCustomInt1(void) {
	return m_iCustom1;
}
int CSprite::getCustomInt2(void) {
	return m_iCustom2;
}
int CSprite::getCustomInt3(void) {
	return m_iCustom3;
}
int CSprite::getCustomInt4(void) {
	return m_iCustom4;
}
int CSprite::getCustomInt5(void) {
	return m_iCustom5;
}


// Custom Floats.
void CSprite::setCustomFloat1(float value){
	m_fCustom1 = value;
}
void CSprite::setCustomFloat2(float value){
	m_fCustom2 = value;
}
void CSprite::setCustomFloat3(float value){
	m_fCustom3 = value;
}
void CSprite::setCustomFloat4(float value){
	m_fCustom4 = value;
}
void CSprite::setCustomFloat5(float value){
	m_fCustom5 = value;
}
float CSprite::getCustomFloat1(void) {
	return m_fCustom1;
}
float CSprite::getCustomFloat2(void) {
	return m_fCustom2;
}
float CSprite::getCustomFloat3(void) {
	return m_fCustom3;
}
float CSprite::getCustomFloat4(void) {
	return m_fCustom4;
}
float CSprite::getCustomFloat5(void) {
	return m_fCustom5;
}

void CSprite::animationUpdate(void) {

	// Is there an animation at all, we need at least two frames to make it an animation...
	if (m_iAnimationFramesInAnimation < 2) {
		return;
	}
	
	// Is is time yet ???
	m_iAnimationFrameDelayCounter--;
	if (m_iAnimationFrameDelayCounter > -1) {
		return;
	}
	// Reset counter.
	m_iAnimationFrameDelayCounter = m_iAnimationFrameDelay;
	
	// Next frame...
	m_iAnimationFramePos += m_iAnimationDirectionOfFrame;
	
	// If it less than zero, it is obviously a back and forth animation.
	// Just set frame to 1 and change direction.
	if (m_iAnimationFramePos<0) {
		m_iAnimationFramePos = 1;
		m_iAnimationDirectionOfFrame = 1;
	}

	// Did we reach max amount of frames???
	if (m_iAnimationFramePos >= m_iAnimationFramesInAnimation) {
		// Do we stop after last frame ???
		if ( m_iAnimationFrameRepeat == 0) {
			// Disable animation and exit...
			m_iAnimationFramesInAnimation = 0;
			return;
		}
		if ( m_iAnimationFrameRepeatBackAndForth ) {
			// If it is back and forth direction, reverse direction and subtract two frames.
			m_iAnimationFramePos = m_iAnimationFramePos -2;
			m_iAnimationDirectionOfFrame = -1;
		} else {
			// If it is not back and forth direction, start over at frame 0.
			m_iAnimationFramePos = 0;
			m_iAnimationDirectionOfFrame = 1;
		}
	}
	
	if (m_iAnimationDirectionTexture == 0) {
		setTexturePos(m_iAnimationFramePos, 0);
	} else {
		setTexturePos(0, m_iAnimationFramePos);
	}

	/*
	int m_iAnimationDirectionTexture;
	int m_iAnimationFramesInAnimation;
	int m_iAnimationDirectionOfFrame;
	int m_iAnimationFramePos;
	int m_iAnimationFrameRepeat;
	int m_iAnimationFrameRepeatBackAndForth;
	int m_iAnimationFrameDelay;
	int m_iAnimationFrameDelayCounter;
	*/
	
}

void CSprite::animationSetTextureDirection(int dir) {
	m_iAnimationDirectionTexture = dir;
}

void CSprite::animationSetFramesInAnimation(int frames) {
	m_iAnimationFramesInAnimation = frames;
}

void CSprite::animationSetFramePos(int pos) {
	m_iAnimationFramePos = pos;
}

void CSprite::animationSetFrameRepeat(int loop) {
	m_iAnimationFrameRepeat = loop;
}

void CSprite::animationSetFrameRepeatBackAndForth(int backAndForth) {
	m_iAnimationFrameRepeatBackAndForth = backAndForth;
}


void CSprite::animationSetFrameDelay(int delay) {
	m_iAnimationFrameDelay = delay;
	m_iAnimationFrameDelayCounter = delay;
}
