#ifndef __CSPRITE_H__
#define __CSPRITE_H__

#include <sps2lib.h>
#include <sps2tags.h>
#include <sps2regstructs.h>
#include <assert.h>
#include "PS2Defines.h"
#include "AddRegisters.h"
#include <math.h>

class CSprite
{
public:
	CSprite( float xPos, float yPos, int xTexture, int yTexture, int widthTexture, int heightTexture);
	~CSprite();

	void CSprite::setPos(float xPos, float yPos);
	void CSprite::setAngle(float Angle);
	void CSprite::addAngle(float Angle);
	void CSprite::setPosAll(int xPos1, int yPos1, int xPos2, int yPos2, int xPos3, int yPos3, int xPos4, int yPos4);
	void CSprite::setTexturePosST(float s1, float s2, float t1, float t2);
	void CSprite::setTexturePosUV(int u1, int u2, int v1, int v2);
	void CSprite::movePos(float xPos, float yPos);
	void CSprite::moveDirection(float direction);
	void CSprite::setVisible(bool isVisible);
	void CSprite::setColor(int Red, int Green, int Blue, int Alpha);
	void CSprite::setTexturePos(int xTexturePos, int yTexturePos);
	void CSprite::setScaleX(float scaleX);
	void CSprite::setScaleY(float scaleY);

	bool CSprite::getVisible(void);
	float CSprite::getXPos(void);
	float CSprite::getYPos(void);
	float CSprite::getAngle(void);
	float CSprite::getSize(void);
	float CSprite::getScaleX(void);
	float CSprite::getScaleY(void);
	int CSprite::getXPos1(void);
	int CSprite::getYPos1(void);
	int CSprite::getXPos2(void);
	int CSprite::getYPos2(void);
	int CSprite::getXPos3(void);
	int CSprite::getYPos3(void);
	int CSprite::getXPos4(void);
	int CSprite::getYPos4(void);
	bool CSprite::inside(CSprite inSprite);

	void CSprite::addSpriteToMem(char *& pMem, int Zvalue);
	void CSprite::addSpriteToGIFTag(char *& pMem, int Zvalue);
	void CSprite::AddGIFTag(char *& pMem, int numSprites);

	void CSprite::setCustomInt1(int value);
	void CSprite::setCustomInt2(int value);
	void CSprite::setCustomInt3(int value);
	void CSprite::setCustomInt4(int value);
	void CSprite::setCustomInt5(int value);
	void CSprite::setCustomFloat1(float value);
	void CSprite::setCustomFloat2(float value);
	void CSprite::setCustomFloat3(float value);
	void CSprite::setCustomFloat4(float value);
	void CSprite::setCustomFloat5(float value);

	int CSprite::getCustomInt1(void);
	int CSprite::getCustomInt2(void);
	int CSprite::getCustomInt3(void);
	int CSprite::getCustomInt4(void);
	int CSprite::getCustomInt5(void);
	float CSprite::getCustomFloat1(void);
	float CSprite::getCustomFloat2(void);
	float CSprite::getCustomFloat3(void);
	float CSprite::getCustomFloat4(void);
	float CSprite::getCustomFloat5(void);
	
	void CSprite::animationUpdate(void);
	void CSprite::animationSetTextureDirection(int dir);
	void CSprite::animationSetFramesInAnimation(int frames);
	void CSprite::animationSetFramePos(int pos);
	void CSprite::animationSetFrameRepeat(int loop);
	void CSprite::animationSetFrameRepeatBackAndForth(int backAndForth);
	void CSprite::animationSetFrameDelay(int delay);
	

private:
	int m_width, m_height;
	int m_xTexture, m_yTexture;
	float m_xPos, m_yPos;
	int m_xPos1, m_yPos1;
	int m_xPos2, m_yPos2;
	int m_xPos3, m_yPos3;
	int m_xPos4, m_yPos4;
	int m_Red, m_Green, m_Blue, m_Alpha;
	bool m_isVisible;
	float m_s1, m_s2, m_t1, m_t2;
	float m_Angle;
	float m_fHalfW, m_fHalfH;
	float m_fSize;
	float m_fScaleX, m_fScaleY;

	int m_iCustom1;
	int m_iCustom2;
	int m_iCustom3;
	int m_iCustom4;
	int m_iCustom5;
	float m_fCustom1;
	float m_fCustom2;
	float m_fCustom3;
	float m_fCustom4;
	float m_fCustom5;

	int m_iAnimationDirectionTexture;
	int m_iAnimationFramesInAnimation;
	int m_iAnimationDirectionOfFrame;
	int m_iAnimationFramePos;
	int m_iAnimationFrameRepeat;
	int m_iAnimationFrameRepeatBackAndForth;
	int m_iAnimationFrameDelay;
	int m_iAnimationFrameDelayCounter;

};

#endif
