#ifndef __CTEXTURE_H__
#define __CTEXTURE_H__

#include <sps2lib.h>
#include <sps2tags.h>
#include <sps2regstructs.h>
#include <assert.h>
#include "PS2Defines.h"
#include "AddRegisters.h"
#include <memory.h>

// All the following is the layout of a bitmap file header (the bit before
// the actual image data). The __attribute__((packed)) tells the compiler
// not to insert any annoying padding. (If it was padded we wouldn't be
// able to read it in in one go easily)
#define BITMAP_ID      0x4D42

struct BitmapFileHeader
{
	uint16  bfType;
    uint32  bfSize;
    uint16  bfReserved1;
    uint16  bfReserved2;
    uint32  bfOffBits;
} __attribute__((packed));

struct BitmapInfoHeader
{
	uint32	biSize;
	int32	biWidth;
	int32	biHeight;
	uint16	biPlanes;
	uint16	biBitCount;
	uint32	biCompression;
	uint32	biSizeImage;
	int32	biXPelsPerMeter;
	int32	biYPelsPerMeter;
	uint32	biClrUsed;
	uint32	biClrImportant;
} __attribute__((packed));

struct BitmapFile
{
	BitmapFileHeader  bitmapfileheader;
	BitmapInfoHeader  bitmapinfoheader;
} __attribute__((packed));

class CTexture
{
public:
	CTexture(int iSPS2Desc);
	~CTexture();

	bool CTexture::LoadBitmap(const char * strFilename);
	bool CTexture::LoadBitmap(const char * strFilename, bool bBlackTransparent, bool bRedAsAlpha);
	bool CTexture::LoadBitmap(const char * strFilename, int iRed, int iGreen, int iBlue);
	bool CTexture::LoadBitmap(const char * strFilename, bool bBlackTransparent, bool bRedAsAlpha, int iRed, int iGreen, int iBlue);
	void CTexture::Upload(uint32 uGSPage, bool bRotate = true);

private:
	uint32 CTexture::TextureLog2(uint32 n);
	bool CTexture::AllocateMemory(int Memory_Size);
	void CTexture::BuildDMA(uint32 uGSPage, bool bRotate);

	char * m_pMemory;
	char * m_pDMAMem;
	int m_iClutOffset;
	int m_iWidth, m_iHeight;
	int m_l2Width, m_l2Height;
	bool m_bHasCLUT;
	bool m_bHasDMA;
	bool m_bRotate;
	int m_iRed;
	int m_iGreen;
	int m_iBlue;
	int m_iSPS2Desc;
	sps2Memory_t * m_pMemCached;
	uint32 m_uGSPage;

};

#endif